@@ -7,7 +7,7 @@ no warnings 'uninitialized';
 
 use Carp;
 
-our $VERSION = '0.01';
+our $VERSION = '0.02';
 
 our %MULTI;
 our %MULTIPARAM;
@@ -68,6 +68,7 @@ sub make_wrapper {
     };
 }
 
+# exports a multimethod with a given name and arguments
 sub multi {
     if (_internal_multi(@_)) {
         croak "Usage: multi name => (Arg1, Arg2, ...) => sub { code };";
@@ -139,6 +140,8 @@ package Class::Multimethods::Pure::Type;
 use Carp;
 use Scalar::Util qw<blessed>;
 
+# The promote multimethod is where the logic is to turn the string "Foo::Bar"
+# into a Type::Package object.
 our $PROMOTE = Class::Multimethods::Pure::Method->new;
 
 sub promote {
@@ -147,25 +150,31 @@ sub promote {
 }
 
 {
-    my $pkg = sub { "Class::Multimethods::Pure::Type::$_[0]"->new(@_[1..$#_]) };
-    $PROMOTE->add_variant(
-        [ $pkg->('Subtype', $pkg->('Any'), sub { blessed $_[0] }) ] => sub {
-            $_[0];
-    });
+    # I put each subtype into a variable so that you can extend the subtypes easily.
     
+    my $pkg = sub { "Class::Multimethods::Pure::Type::$_[0]"->new(@_[1..$#_]) };
+
+    # Anything that is blessed is probably already a Type object
+    our $PROMOTE_BLESSED = $pkg->('Subtype', $pkg->('Any'), sub { blessed $_[0] });
+    $PROMOTE->add_variant([ $PROMOTE_BLESSED ] => sub { $_[0] });
+    
+    # ARRAY, HASH, etc. get an Unblessed type for unblessed references.
+    our $PROMOTE_UNBLESSED = $pkg->('Subtype', $pkg->('Any'),
+                                sub { Class::Multimethods::Pure::Type::Unblessed->is_unblessed($_[0]) });
     $PROMOTE->add_variant(
-        [ $pkg->('Subtype', $pkg->('Any'), 
-            sub { Class::Multimethods::Pure::Type::Unblessed->is_unblessed($_[0]) }) ]
-        => sub { 
+        [ $PROMOTE_UNBLESSED ] => sub { 
             Class::Multimethods::Pure::Type::Unblessed->new($_[0])
     });
 
+    # Anything else gets turned into a package.
     $PROMOTE->add_variant(
         [ $pkg->('Any') ] => sub {
             Class::Multimethods::Pure::Type::Package->new($_[0])
     });
 }
 
+# The subset multimethod is the most important multi used in the core.  It
+# determines whether the left class is a subset of the right class.
 our $SUBSET = Class::Multimethods::Pure::Method->new;
 
 sub subset {
@@ -173,11 +182,9 @@ sub subset {
     $SUBSET->call($self, $other);
 }
 
-our $EQUAL = Class::Multimethods::Pure::Method->new;
-
 sub equal {
     my ($self, $other) = @_;
-    $EQUAL->call($self, $other);
+    subset($self, $other) && subset($other, $self);
 }
 
 sub matches;
@@ -225,11 +232,11 @@ sub string;
     $SUBSET->add_variant(
         [ $pkg->('Type::Subtypable'), $pkg->('Type::Subtype') ] => sub { 0 });
 
-    our $indent = 0;
     $SUBSET->add_variant(
         [ $pkg->('Type::Subtype'), $pkg->('Type::Subtype') ] => sub {
             my ($a, $b) = @_;
-            $a->equal($b) || $a->base->subset($b);
+            $a->base->subset($b) || 
+                $a->base->subset($b->base) && $a->condition == $b->condition;
         });
     
     $SUBSET->add_variant(
@@ -250,57 +257,6 @@ sub string;
              # just like (Junction, Type)
              $a->logic(map { $_->subset($b) } $a->values);
      });
-
-    #############
-     
-    $EQUAL->add_variant(
-        [ $pkg->('Type'), $pkg->('Type') ] => sub {
-            0;
-    });
-
-    # If you change this, you should also change the bootstrap, Type::Package::equal.
-    $EQUAL->add_variant(
-        [ $pkg->('Type::Package'), $pkg->('Type::Package') ] => sub {
-            my ($a, $b) = @_;
-            $a->name eq $b->name;
-    });
-
-    $EQUAL->add_variant(
-        [ $pkg->('Type::Unblessed'), $pkg->('Type::Unblessed') ] => sub {
-            my ($a, $b) = @_;
-            $a->name eq $b->name;
-    });
-
-    $EQUAL->add_variant(
-        [ $pkg->('Type::Any'), $pkg->('Type::Any') ] => sub { 1 });
-
-    $EQUAL->add_variant(
-        [ $pkg->('Type::Subtype'), $pkg->('Type::Subtype') ] => sub {
-            my ($a, $b) = @_;
-            $a->base == $b->base && $a->condition == $b->condition;
-    });
-
-    my $jequal = sub {
-        my ($a, $b) = @_;
-        !$a->logic(map { !$_->subset($b) } $a->values)
-        && $a->logic(map { $b->subset($_) } $a->values);
-    };
-
-    
-    $EQUAL->add_variant(
-        [ $pkg->('Type::Junction'), $pkg->('Type') ] => sub {
-            $jequal->($_[0], $_[1]);
-    });
-
-    $EQUAL->add_variant(
-        [ $pkg->('Type'), $pkg->('Type::Junction') ] => sub {
-            $jequal->($_[1], $_[0]);
-    });
-
-    $EQUAL->add_variant(
-        [ $pkg->('Type::Junction'), $pkg->('Type::Junction') ] => sub {
-            $jequal->($_[0], $_[1]);
-    });
 }
 
 
@@ -332,18 +288,6 @@ sub subset {
     }
 }
 
-# Again, bootstrapping.
-sub equal {
-    my ($self, $other) = @_;
-
-    if (ref $self eq __PACKAGE__ && ref $other eq __PACKAGE__) {
-        $self->name eq $other->name;
-    }
-    else {
-        $self->SUPER::equal($other);
-    }
-}
-
 sub name {
     my ($self) = @_;
     $self->{name};
@@ -531,6 +475,10 @@ sub string {
 }
 
 package Class::Multimethods::Pure::Type::Injunction;
+# The none() type has some very, very strange behavior when you think
+# about it.  In particular, note that none() (with no arguments) is
+# at both the top and bottom of the type lattice.  Perhaps none()
+# should not be allowed, or should require arguments.
 
 # A none type
 use base 'Class::Multimethods::Pure::Type::Junction';
@@ -583,7 +531,7 @@ sub less {
         my $cmp = $args[$i]->subset($brgs[$i]);
         return 0 unless $cmp;
         if ($cmp && !$proper) {
-            $proper = !$args[$i]->equal($brgs[$i]);
+            $proper = !$brgs[$i]->subset($args[$i]);
         }
     }
 
@@ -695,7 +643,7 @@ sub call {
 
 1;
 
-=head1 TITLE
+=head1 NAME
 
 Class::Multimethods::Pure - Method-ordered multimethod dispatch
 
@@ -1070,6 +1018,76 @@ Then the upper one (Junction in this case) will get threaded first,
 because a Junction is not a KunctionObject, so it doesn't fit in the
 latter three methods.
 
+=head2 Extending
+
+Class::Multimethods::Pure was written to be extended in many ways, but
+with a focus on adding new types of, er, types.  Let's say you want to
+add Perl 6-ish roles to the Class::Multimethods::Pure dispatcher.  You
+need to do four things:
+
+=over
+
+=item * 
+
+Create a class, say My::Role derived from
+Class::Multimethods::Pure::Type.  
+
+=item * 
+
+Define the method My::Role::matches, which takes a scalar and returns
+whether it is a member of that class (including subclasses, etc.).
+
+=item *
+
+Define the method My::Role::string, which returns a reasonable string
+representation of the type, for the user's sake.
+
+=item *
+
+Define as many multimethod variants of "subset" as necessary, which
+return whether an object which is a member of the left type implies that
+it is a member of the right type.  Construct a
+Class::Multimethods::Pure::Type::Package type for your type for the
+multimethod.  For a role, you'd need to define:
+
+    $Class::Multimethods::Pure::Type::SUBSET->add_variant(
+        [ Class::Multimethods::Pure::Type::Package->new('My::Role'),
+          Class::Multimethods::Pure::Type::Package->new('My::Role') ] =>
+        sub {...});
+
+And:
+
+    $Class::Multimethods::Pure::Type::SUBSET->add_variant(
+        [ Class::Multimethods::Pure::Type::Package->new(
+              'Class::Multimethods::Pure::Type::Package'),
+          Class::Multimethods::Pure::Type::Package->new('My::Role') ] =>
+        sub {...});
+
+(Ugh, I wish my module name weren't so long).
+
+=back
+
+After you have defined these, you have fulfilled the
+Class::Multimethods::Pure::Type interface, and now you can pass an
+object of type My::Role to multi() and it will be dispatched using the
+pure-ordered scheme.  It is nice to give the user a concise constructor
+for your object type.
+
+You can also automatically promote strings into objects by defining
+variants on the (unary) multimethod
+$Class::Multimethods::Pure::Type::PROMOTE.  So to promote strings that
+happen to be the names of roles, do:
+
+    $Class::Multimethods::Pure::Type::PROMOTE->add_variant(
+        [ Class::Multimethods::Pure::Type::Subtype->new(
+            Class::Multimethods::Pure::Type::Any->new,
+            sub { is_a_role_name($_[0]) }) 
+        ] => 
+            sub { My::Role->new($_[0]) });
+
+Now when you pass strings to "multi", if is_a_role_name returns true on
+them, they will be promoted to a My::Role object.
+
 =head1 AUTHOR
 
 Luke Palmer <lrpalmer@gmail.com>
